@@ -41,6 +41,7 @@
 
 #include "qwaylandintegration_p.h"
 #include "qwaylandwindow_p.h"
+#include "qwaylandsurface_p.h"
 #include "qwaylandabstractdecoration_p.h"
 #include "qwaylandscreen_p.h"
 #include "qwaylandcursor_p.h"
@@ -50,7 +51,11 @@
 #endif
 #if QT_CONFIG(wayland_datadevice)
 #include "qwaylanddatadevicemanager_p.h"
-#endif
+#include "qwaylanddatadevice_p.h"
+#endif // QT_CONFIG(wayland_datadevice)
+#if QT_CONFIG(wayland_client_primary_selection)
+#include "qwaylandprimaryselectionv1_p.h"
+#endif // QT_CONFIG(wayland_client_primary_selection)
 #if QT_CONFIG(cursor)
 #include <wayland-cursor.h>
 #endif
@@ -64,13 +69,16 @@
 #include "qwaylandextendedsurface_p.h"
 #include "qwaylandsubsurface_p.h"
 #include "qwaylandtouch_p.h"
+#include "qwaylandtabletv2_p.h"
 #include "qwaylandqtkey_p.h"
 
 #include <QtWaylandClient/private/qwayland-text-input-unstable-v2.h>
+#include <QtWaylandClient/private/qwayland-wp-primary-selection-unstable-v1.h>
 
 #include <QtCore/private/qcore_unix_p.h>
 
 #include <QtCore/QAbstractEventDispatcher>
+#include <QtGui/qpa/qwindowsysteminterface.h>
 #include <QtGui/private/qguiapplication_p.h>
 
 #include <QtCore/QDebug>
@@ -107,7 +115,11 @@ struct ::wl_region *QWaylandDisplay::createRegion(const QRegion &qregion)
         return nullptr;
     }
 
-    return mSubCompositor->get_subsurface(window->object(), parent->object());
+    // Make sure we don't pass NULL surfaces to libwayland (crashes)
+    Q_ASSERT(parent->wlSurface());
+    Q_ASSERT(window->wlSurface());
+
+    return mSubCompositor->get_subsurface(window->wlSurface(), parent->wlSurface());
 }
 
 QWaylandShellIntegration *QWaylandDisplay::shellIntegration() const
@@ -141,7 +153,18 @@ QWaylandDisplay::QWaylandDisplay(QWaylandIntegration *waylandIntegration)
 
     mWindowManagerIntegration.reset(new QWaylandWindowManagerIntegration(this));
 
+#if QT_CONFIG(xkbcommon)
+    mXkbContext.reset(xkb_context_new(XKB_CONTEXT_NO_FLAGS));
+    if (!mXkbContext)
+        qCWarning(lcQpaWayland, "failed to create xkb context");
+#endif
+
     forceRoundTrip();
+
+    if (!mWaitingScreens.isEmpty()) {
+        // Give wl_output.done and zxdg_output_v1.done events a chance to arrive
+        forceRoundTrip();
+    }
 }
 
 QWaylandDisplay::~QWaylandDisplay(void)
@@ -149,24 +172,35 @@ QWaylandDisplay::~QWaylandDisplay(void)
     if (mSyncCallback)
         wl_callback_destroy(mSyncCallback);
 
-    qDeleteAll(mInputDevices);
-    mInputDevices.clear();
+    qDeleteAll(qExchange(mInputDevices, {}));
 
-    foreach (QWaylandScreen *screen, mScreens) {
-        mWaylandIntegration->destroyScreen(screen);
+    for (QWaylandScreen *screen : qExchange(mScreens, {})) {
+        QWindowSystemInterface::handleScreenRemoved(screen);
     }
-    mScreens.clear();
+    qDeleteAll(mWaitingScreens);
 
 #if QT_CONFIG(wayland_datadevice)
     delete mDndSelectionHandler.take();
 #endif
 #if QT_CONFIG(cursor)
-    qDeleteAll(mCursorThemesBySize);
+    qDeleteAll(mCursorThemes);
 #endif
     if (mDisplay)
         wl_display_disconnect(mDisplay);
 }
 
+void QWaylandDisplay::ensureScreen()
+{
+    if (!mScreens.empty() || mPlaceholderScreen)
+        return; // There are real screens or we already have a fake one
+
+    qCInfo(lcQpaWayland) << "Creating a fake screen in order for Qt not to crash";
+
+    mPlaceholderScreen = new QPlatformPlaceholderScreen();
+    QWindowSystemInterface::handleScreenAdded(mPlaceholderScreen);
+    Q_ASSERT(!QGuiApplication::screens().empty());
+}
+
 void QWaylandDisplay::checkError() const
 {
     int ecode = wl_display_get_error(mDisplay);
@@ -187,6 +221,17 @@ void QWaylandDisplay::flushRequests()
     if (wl_display_dispatch_pending(mDisplay) < 0)
         checkError();
 
+    {
+        QReadLocker locker(&m_frameQueueLock);
+        for (const FrameQueue &q : mExternalQueues) {
+            QMutexLocker locker(q.mutex);
+            while (wl_display_prepare_read_queue(mDisplay, q.queue) != 0)
+                wl_display_dispatch_queue_pending(mDisplay, q.queue);
+            wl_display_read_events(mDisplay);
+            wl_display_dispatch_queue_pending(mDisplay, q.queue);
+        }
+    }
+
     wl_display_flush(mDisplay);
 }
 
@@ -196,6 +241,27 @@ void QWaylandDisplay::blockingReadEvents()
         checkError();
 }
 
+void QWaylandDisplay::destroyFrameQueue(const QWaylandDisplay::FrameQueue &q)
+{
+    QWriteLocker locker(&m_frameQueueLock);
+    auto it = std::find_if(mExternalQueues.begin(),
+                           mExternalQueues.end(),
+                           [&q] (const QWaylandDisplay::FrameQueue &other){ return other.queue == q.queue; });
+    Q_ASSERT(it != mExternalQueues.end());
+    mExternalQueues.erase(it);
+    if (q.queue != nullptr)
+        wl_event_queue_destroy(q.queue);
+    delete q.mutex;
+}
+
+QWaylandDisplay::FrameQueue QWaylandDisplay::createFrameQueue()
+{
+    QWriteLocker locker(&m_frameQueueLock);
+    FrameQueue q{createEventQueue()};
+    mExternalQueues.append(q);
+    return q;
+}
+
 wl_event_queue *QWaylandDisplay::createEventQueue()
 {
     return wl_display_create_queue(mDisplay);
@@ -232,14 +298,26 @@ void QWaylandDisplay::dispatchQueueWhile(wl_event_queue *queue, std::function<bo
 
 QWaylandScreen *QWaylandDisplay::screenForOutput(struct wl_output *output) const
 {
-    for (int i = 0; i < mScreens.size(); ++i) {
-        QWaylandScreen *screen = static_cast<QWaylandScreen *>(mScreens.at(i));
+    for (auto screen : qAsConst(mScreens)) {
         if (screen->output() == output)
             return screen;
     }
     return nullptr;
 }
 
+void QWaylandDisplay::handleScreenInitialized(QWaylandScreen *screen)
+{
+    if (!mWaitingScreens.removeOne(screen))
+        return;
+    mScreens.append(screen);
+    QWindowSystemInterface::handleScreenAdded(screen);
+    if (mPlaceholderScreen) {
+        QWindowSystemInterface::handleScreenRemoved(mPlaceholderScreen);
+        // handleScreenRemoved deletes the platform screen
+        mPlaceholderScreen = nullptr;
+    }
+}
+
 void QWaylandDisplay::waitForScreens()
 {
     flushRequests();
@@ -261,16 +339,10 @@ void QWaylandDisplay::waitForScreens()
 
 void QWaylandDisplay::registry_global(uint32_t id, const QString &interface, uint32_t version)
 {
-    Q_UNUSED(version);
-
     struct ::wl_registry *registry = object();
 
     if (interface == QStringLiteral("wl_output")) {
-        QWaylandScreen *screen = new QWaylandScreen(this, version, id);
-        mScreens.append(screen);
-        // We need to get the output events before creating surfaces
-        forceRoundTrip();
-        mWaylandIntegration->screenAdded(screen);
+        mWaitingScreens << new QWaylandScreen(this, version, id);
     } else if (interface == QStringLiteral("wl_compositor")) {
         mCompositorVersion = qMin((int)version, 3);
         mCompositor.init(registry, id, mCompositorVersion);
@@ -291,26 +363,36 @@ void QWaylandDisplay::registry_global(uint32_t id, const QString &interface, uin
         mTouchExtension.reset(new QWaylandTouchExtension(this, id));
     } else if (interface == QStringLiteral("zqt_key_v1")) {
         mQtKeyExtension.reset(new QWaylandQtKeyExtension(this, id));
-    } else if (interface == QStringLiteral("zwp_text_input_manager_v2")) {
+    } else if (interface == QStringLiteral("zwp_tablet_manager_v2")) {
+        mTabletManager.reset(new QWaylandTabletManagerV2(this, id, qMin(1, int(version))));
+#if QT_CONFIG(wayland_client_primary_selection)
+    } else if (interface == QStringLiteral("zwp_primary_selection_device_manager_v1")) {
+        mPrimarySelectionManager.reset(new QWaylandPrimarySelectionDeviceManagerV1(this, id, 1));
+#endif
+    } else if (interface == QStringLiteral("zwp_text_input_manager_v2") && !mClientSideInputContextRequested) {
         mTextInputManager.reset(new QtWayland::zwp_text_input_manager_v2(registry, id, 1));
-        foreach (QWaylandInputDevice *inputDevice, mInputDevices) {
+        for (QWaylandInputDevice *inputDevice : qAsConst(mInputDevices))
             inputDevice->setTextInput(new QWaylandTextInput(this, mTextInputManager->get_text_input(inputDevice->wl_seat())));
-        }
+        mWaylandIntegration->reconfigureInputContext();
     } else if (interface == QStringLiteral("qt_hardware_integration")) {
-        mHardwareIntegration.reset(new QWaylandHardwareIntegration(registry, id));
-        // make a roundtrip here since we need to receive the events sent by
-        // qt_hardware_integration before creating windows
-        forceRoundTrip();
+        bool disableHardwareIntegration = qEnvironmentVariableIntValue("QT_WAYLAND_DISABLE_HW_INTEGRATION");
+        if (!disableHardwareIntegration) {
+            mHardwareIntegration.reset(new QWaylandHardwareIntegration(registry, id));
+            // make a roundtrip here since we need to receive the events sent by
+            // qt_hardware_integration before creating windows
+            forceRoundTrip();
+        }
     } else if (interface == QLatin1String("zxdg_output_manager_v1")) {
-        mXdgOutputManager.reset(new QtWayland::zxdg_output_manager_v1(registry, id, 1));
-        for (auto *screen : qAsConst(mScreens))
+        mXdgOutputManager.reset(new QWaylandXdgOutputManagerV1(this, id, version));
+        for (auto *screen : qAsConst(mWaitingScreens))
             screen->initXdgOutput(xdgOutputManager());
         forceRoundTrip();
     }
 
     mGlobals.append(RegistryGlobal(id, interface, version, registry));
 
-    foreach (Listener l, mRegistryListeners)
+    const auto copy = mRegistryListeners; // be prepared for listeners unregistering on notification
+    for (Listener l : copy)
         (*l.listener)(l.data, registry, id, interface, version);
 }
 
@@ -320,13 +402,29 @@ void QWaylandDisplay::registry_global_remove(uint32_t id)
         RegistryGlobal &global = mGlobals[i];
         if (global.id == id) {
             if (global.interface == QStringLiteral("wl_output")) {
-                foreach (QWaylandScreen *screen, mScreens) {
+                for (auto *screen : mWaitingScreens) {
                     if (screen->outputId() == id) {
-                        mScreens.removeOne(screen);
-                        mWaylandIntegration->destroyScreen(screen);
+                        mWaitingScreens.removeOne(screen);
+                        delete screen;
                         break;
                     }
                 }
+
+                for (QWaylandScreen *screen : qAsConst(mScreens)) {
+                    if (screen->outputId() == id) {
+                        mScreens.removeOne(screen);
+                        // If this is the last screen, we have to add a fake screen, or Qt will break.
+                        ensureScreen();
+                        QWindowSystemInterface::handleScreenRemoved(screen);
+                        break;
+                    }
+                }
+            }
+            if (global.interface == QStringLiteral("zwp_text_input_manager_v2")) {
+                mTextInputManager.reset();
+                for (QWaylandInputDevice *inputDevice : qAsConst(mInputDevices))
+                    inputDevice->setTextInput(nullptr);
+                mWaylandIntegration->reconfigureInputContext();
             }
             mGlobals.removeAt(i);
             break;
@@ -334,9 +432,9 @@ void QWaylandDisplay::registry_global_remove(uint32_t id)
     }
 }
 
-bool QWaylandDisplay::hasRegistryGlobal(const QString &interfaceName)
+bool QWaylandDisplay::hasRegistryGlobal(QStringView interfaceName) const
 {
-    Q_FOREACH (const RegistryGlobal &global, mGlobals)
+    for (const RegistryGlobal &global : mGlobals)
         if (global.interface == interfaceName)
             return true;
 
@@ -529,40 +627,20 @@ QWaylandInputDevice *QWaylandDisplay::defaultInputDevice() const
 
 #if QT_CONFIG(cursor)
 
-void QWaylandDisplay::setCursor(struct wl_buffer *buffer, struct wl_cursor_image *image, qreal dpr)
+QWaylandCursor *QWaylandDisplay::waylandCursor()
 {
-    /* Qt doesn't tell us which input device we should set the cursor
-     * for, so set it for all devices. */
-    for (int i = 0; i < mInputDevices.count(); i++) {
-        QWaylandInputDevice *inputDevice = mInputDevices.at(i);
-        inputDevice->setCursor(buffer, image, dpr);
-    }
+    if (!mCursor)
+        mCursor.reset(new QWaylandCursor(this));
+    return mCursor.data();
 }
 
-void QWaylandDisplay::setCursor(const QSharedPointer<QWaylandBuffer> &buffer, const QPoint &hotSpot, qreal dpr)
+QWaylandCursorTheme *QWaylandDisplay::loadCursorTheme(const QString &name, int pixelSize)
 {
-    /* Qt doesn't tell us which input device we should set the cursor
-     * for, so set it for all devices. */
-    for (int i = 0; i < mInputDevices.count(); i++) {
-        QWaylandInputDevice *inputDevice = mInputDevices.at(i);
-        inputDevice->setCursor(buffer, hotSpot, dpr);
-    }
-}
-
-QWaylandCursorTheme *QWaylandDisplay::loadCursorTheme(qreal devicePixelRatio)
-{
-    constexpr int defaultCursorSize = 32;
-    static const int xCursorSize = qEnvironmentVariableIntValue("XCURSOR_SIZE");
-    int cursorSize = xCursorSize > 0 ? xCursorSize : defaultCursorSize;
-
-    if (compositorVersion() >= 3) // set_buffer_scale is not supported on earlier versions
-        cursorSize *= devicePixelRatio;
-
-    if (auto *theme = mCursorThemesBySize.value(cursorSize, nullptr))
+    if (auto *theme = mCursorThemes.value({name, pixelSize}, nullptr))
         return theme;
 
-    if (auto *theme = QWaylandCursorTheme::create(shm(), cursorSize)) {
-        mCursorThemesBySize[cursorSize] = theme;
+    if (auto *theme = QWaylandCursorTheme::create(shm(), pixelSize, name)) {
+        mCursorThemes[{name, pixelSize}] = theme;
         return theme;
     }
 
@@ -571,6 +649,6 @@ QWaylandCursorTheme *QWaylandDisplay::loadCursorTheme(qreal devicePixelRatio)
 
 #endif // QT_CONFIG(cursor)
 
-}
+} // namespace QtWaylandClient
 
 QT_END_NAMESPACE
