@@ -86,42 +86,23 @@
 #include "qwaylandinputdeviceintegration_p.h"
 #include "qwaylandinputdeviceintegrationfactory_p.h"
 
+#if QT_CONFIG(accessibility_atspi_bridge)
+#include <QtLinuxAccessibilitySupport/private/bridge_p.h>
+#endif
+
+#if QT_CONFIG(xkbcommon)
+#include <QtXkbCommonSupport/private/qxkbcommon_p.h>
+#endif
+
+#if QT_CONFIG(vulkan)
+#include "qwaylandvulkaninstance_p.h"
+#include "qwaylandvulkanwindow_p.h"
+#endif
+
 QT_BEGIN_NAMESPACE
 
 namespace QtWaylandClient {
 
-class GenericWaylandTheme: public QGenericUnixTheme
-{
-public:
-    static QStringList themeNames()
-    {
-        QStringList result;
-
-        if (QGuiApplication::desktopSettingsAware()) {
-            const QByteArray desktopEnvironment = QGuiApplicationPrivate::platformIntegration()->services()->desktopEnvironment();
-
-            if (desktopEnvironment == QByteArrayLiteral("KDE")) {
-#if QT_CONFIG(settings)
-                result.push_back(QStringLiteral("kde"));
-#endif
-            } else if (!desktopEnvironment.isEmpty() &&
-                desktopEnvironment != QByteArrayLiteral("UNKNOWN") &&
-                desktopEnvironment != QByteArrayLiteral("GNOME") &&
-                desktopEnvironment != QByteArrayLiteral("UNITY") &&
-                desktopEnvironment != QByteArrayLiteral("MATE") &&
-                desktopEnvironment != QByteArrayLiteral("XFCE") &&
-                desktopEnvironment != QByteArrayLiteral("LXDE"))
-                // Ignore X11 desktop environments
-                result.push_back(QString::fromLocal8Bit(desktopEnvironment.toLower()));
-        }
-
-        if (result.isEmpty())
-            result.push_back(QLatin1String(QGenericUnixTheme::name));
-
-        return result;
-    }
-};
-
 QWaylandIntegration::QWaylandIntegration()
 #if defined(Q_OS_MACOS)
     : mFontDb(new QCoreTextFontDatabaseEngineFactory<QCoreTextFontEngine>)
@@ -129,9 +110,6 @@ QWaylandIntegration::QWaylandIntegration()
     : mFontDb(new QGenericUnixFontDatabase())
 #endif
     , mNativeInterface(new QWaylandNativeInterface(this))
-#if QT_CONFIG(accessibility)
-    , mAccessibility(new QPlatformAccessibility())
-#endif
 {
     initializeInputDeviceIntegration();
     mDisplay.reset(new QWaylandDisplay(this));
@@ -145,20 +123,8 @@ QWaylandIntegration::QWaylandIntegration()
 #if QT_CONFIG(draganddrop)
     mDrag.reset(new QWaylandDrag(mDisplay.data()));
 #endif
-    QString icStr = QPlatformInputContextFactory::requested();
-    if (!icStr.isNull()) {
-        mInputContext.reset(QPlatformInputContextFactory::create(icStr));
-    } else {
-        //try to use the input context using the wl_text_input interface
-        QPlatformInputContext *ctx = new QWaylandInputContext(mDisplay.data());
-        mInputContext.reset(ctx);
 
-        //use the traditional way for on screen keyboards for now
-        if (!mInputContext.data()->isValid()) {
-            ctx = QPlatformInputContextFactory::create();
-            mInputContext.reset(ctx);
-        }
-    }
+    reconfigureInputContext();
 }
 
 QWaylandIntegration::~QWaylandIntegration()
@@ -197,7 +163,12 @@ QPlatformWindow *QWaylandIntegration::createPlatformWindow(QWindow *window) cons
         && mDisplay->clientBufferIntegration())
         return mDisplay->clientBufferIntegration()->createEglWindow(window);
 
-    return new QWaylandShmWindow(window);
+#if QT_CONFIG(vulkan)
+    if (window->surfaceType() == QSurface::VulkanSurface)
+        return new QWaylandVulkanWindow(window, mDisplay.data());
+#endif // QT_CONFIG(vulkan)
+
+    return new QWaylandShmWindow(window, mDisplay.data());
 }
 
 #if QT_CONFIG(opengl)
@@ -211,7 +182,7 @@ QPlatformOpenGLContext *QWaylandIntegration::createPlatformOpenGLContext(QOpenGL
 
 QPlatformBackingStore *QWaylandIntegration::createPlatformBackingStore(QWindow *window) const
 {
-    return new QWaylandShmBackingStore(window);
+    return new QWaylandShmBackingStore(window, mDisplay.data());
 }
 
 QAbstractEventDispatcher *QWaylandIntegration::createEventDispatcher() const
@@ -227,12 +198,10 @@ void QWaylandIntegration::initialize()
 
     int fd = wl_display_get_fd(mDisplay->wl_display());
     QSocketNotifier *sn = new QSocketNotifier(fd, QSocketNotifier::Read, mDisplay.data());
-    QObject::connect(sn, SIGNAL(activated(int)), mDisplay.data(), SLOT(flushRequests()));
+    QObject::connect(sn, SIGNAL(activated(QSocketDescriptor)), mDisplay.data(), SLOT(flushRequests()));
 
-    if (mDisplay->screens().isEmpty()) {
-        qWarning() << "Running on a compositor with no screens is not supported";
-        ::exit(EXIT_FAILURE);
-    }
+    // Qt does not support running with no screens
+    mDisplay->ensureScreen();
 }
 
 QPlatformFontDatabase *QWaylandIntegration::fontDatabase() const
@@ -264,19 +233,21 @@ QVariant QWaylandIntegration::styleHint(StyleHint hint) const
     if (hint == ShowIsFullScreen && mDisplay->windowManagerIntegration())
         return mDisplay->windowManagerIntegration()->showIsFullScreen();
 
-    switch (hint) {
-    case QPlatformIntegration::FontSmoothingGamma:
-        return qreal(1.0);
-    default:
-        break;
-    }
-
     return QPlatformIntegration::styleHint(hint);
 }
 
 #if QT_CONFIG(accessibility)
 QPlatformAccessibility *QWaylandIntegration::accessibility() const
 {
+    if (!mAccessibility) {
+#if QT_CONFIG(accessibility_atspi_bridge)
+        Q_ASSERT_X(QCoreApplication::eventDispatcher(), "QWaylandIntegration",
+            "Initializing accessibility without event-dispatcher!");
+        mAccessibility.reset(new QSpiAccessibleBridge());
+#else
+        mAccessibility.reset(new QPlatformAccessibility());
+#endif
+    }
     return mAccessibility.data();
 }
 #endif
@@ -291,16 +262,30 @@ QWaylandDisplay *QWaylandIntegration::display() const
     return mDisplay.data();
 }
 
+QList<int> QWaylandIntegration::possibleKeys(const QKeyEvent *event) const
+{
+    if (auto *seat = mDisplay->currentInputDevice())
+        return seat->possibleKeys(event);
+    return {};
+}
+
 QStringList QWaylandIntegration::themeNames() const
 {
-    return GenericWaylandTheme::themeNames();
+    return QGenericUnixTheme::themeNames();
 }
 
 QPlatformTheme *QWaylandIntegration::createPlatformTheme(const QString &name) const
 {
-    return GenericWaylandTheme::createUnixTheme(name);
+    return QGenericUnixTheme::createUnixTheme(name);
 }
 
+#if QT_CONFIG(vulkan)
+QPlatformVulkanInstance *QWaylandIntegration::createPlatformVulkanInstance(QVulkanInstance *instance) const
+{
+    return new QWaylandVulkanInstance(instance);
+}
+#endif // QT_CONFIG(vulkan)
+
 // May be called from non-GUI threads
 QWaylandClientBufferIntegration *QWaylandIntegration::clientBufferIntegration() const
 {
@@ -335,31 +320,34 @@ void QWaylandIntegration::initializeClientBufferIntegration()
     if (mClientBufferIntegrationInitialized)
         return;
 
-    QString targetKey;
-    bool disableHardwareIntegration = qEnvironmentVariableIsSet("QT_WAYLAND_DISABLE_HW_INTEGRATION");
-    disableHardwareIntegration = disableHardwareIntegration || !mDisplay->hardwareIntegration();
-    if (disableHardwareIntegration) {
-        QByteArray clientBufferIntegrationName = qgetenv("QT_WAYLAND_CLIENT_BUFFER_INTEGRATION");
-        if (clientBufferIntegrationName.isEmpty())
-            clientBufferIntegrationName = QByteArrayLiteral("wayland-egl");
-        targetKey = QString::fromLocal8Bit(clientBufferIntegrationName);
-    } else {
-        targetKey = mDisplay->hardwareIntegration()->clientBufferIntegration();
-        if (targetKey == QLatin1String("wayland-eglstream-controller"))
+    QString targetKey = QString::fromLocal8Bit(qgetenv("QT_WAYLAND_CLIENT_BUFFER_INTEGRATION"));
+
+    if (targetKey.isEmpty()) {
+        if (mDisplay->hardwareIntegration()
+                && mDisplay->hardwareIntegration()->clientBufferIntegration() != QLatin1String("wayland-eglstream-controller")
+                && mDisplay->hardwareIntegration()->clientBufferIntegration() != QLatin1String("linux-dmabuf-unstable-v1")) {
+            targetKey = mDisplay->hardwareIntegration()->clientBufferIntegration();
+        } else {
             targetKey = QLatin1String("wayland-egl");
+        }
     }
 
     if (targetKey.isEmpty()) {
         qWarning("Failed to determine what client buffer integration to use");
     } else {
         QStringList keys = QWaylandClientBufferIntegrationFactory::keys();
-        if (keys.contains(targetKey)) {
+        qCDebug(lcQpaWayland) << "Available client buffer integrations:" << keys;
+
+        if (keys.contains(targetKey))
             mClientBufferIntegration.reset(QWaylandClientBufferIntegrationFactory::create(targetKey, QStringList()));
-        }
-        if (mClientBufferIntegration)
+
+        if (mClientBufferIntegration) {
+            qCDebug(lcQpaWayland) << "Initializing client buffer integration" << targetKey;
             mClientBufferIntegration->initialize(mDisplay.data());
-        else
-            qWarning("Failed to load client buffer integration: %s\n", qPrintable(targetKey));
+        } else {
+            qCWarning(lcQpaWayland) << "Failed to load client buffer integration:" << targetKey;
+            qCWarning(lcQpaWayland) << "Available client buffer integrations:" << keys;
+        }
     }
 
     // This must be set last to make sure other threads don't use the
@@ -371,16 +359,10 @@ void QWaylandIntegration::initializeServerBufferIntegration()
 {
     mServerBufferIntegrationInitialized = true;
 
-    QString targetKey;
+    QString targetKey = QString::fromLocal8Bit(qgetenv("QT_WAYLAND_SERVER_BUFFER_INTEGRATION"));
 
-    bool disableHardwareIntegration = qEnvironmentVariableIsSet("QT_WAYLAND_DISABLE_HW_INTEGRATION");
-    disableHardwareIntegration = disableHardwareIntegration || !mDisplay->hardwareIntegration();
-    if (disableHardwareIntegration) {
-        QByteArray serverBufferIntegrationName = qgetenv("QT_WAYLAND_SERVER_BUFFER_INTEGRATION");
-        targetKey = QString::fromLocal8Bit(serverBufferIntegrationName);
-    } else {
+    if (targetKey.isEmpty() && mDisplay->hardwareIntegration())
         targetKey = mDisplay->hardwareIntegration()->serverBufferIntegration();
-    }
 
     if (targetKey.isEmpty()) {
         qWarning("Failed to determine what server buffer integration to use");
@@ -388,13 +370,18 @@ void QWaylandIntegration::initializeServerBufferIntegration()
     }
 
     QStringList keys = QWaylandServerBufferIntegrationFactory::keys();
-    if (keys.contains(targetKey)) {
+    qCDebug(lcQpaWayland) << "Available server buffer integrations:" << keys;
+
+    if (keys.contains(targetKey))
         mServerBufferIntegration.reset(QWaylandServerBufferIntegrationFactory::create(targetKey, QStringList()));
-    }
-    if (mServerBufferIntegration)
+
+    if (mServerBufferIntegration) {
+        qCDebug(lcQpaWayland) << "Initializing server buffer integration" << targetKey;
         mServerBufferIntegration->initialize(mDisplay.data());
-    else
-        qWarning("Failed to load server buffer integration %s\n", qPrintable(targetKey));
+    } else {
+        qCWarning(lcQpaWayland) << "Failed to load server buffer integration: " <<  targetKey;
+        qCWarning(lcQpaWayland) << "Available server buffer integrations:" << keys;
+    }
 }
 
 void QWaylandIntegration::initializeShellIntegration()
@@ -419,10 +406,10 @@ void QWaylandIntegration::initializeShellIntegration()
         preferredShells << QLatin1String("wl-shell") << QLatin1String("ivi-shell");
     }
 
-    Q_FOREACH (QString preferredShell, preferredShells) {
+    for (const QString &preferredShell : qAsConst(preferredShells)) {
         mShellIntegration.reset(createShellIntegration(preferredShell));
         if (mShellIntegration) {
-            qDebug("Using the '%s' shell integration", qPrintable(preferredShell));
+            qCDebug(lcQpaWayland, "Using the '%s' shell integration", qPrintable(preferredShell));
             break;
         }
     }
@@ -431,6 +418,8 @@ void QWaylandIntegration::initializeShellIntegration()
         qCWarning(lcQpaWayland) << "Loading shell integration failed.";
         qCWarning(lcQpaWayland) << "Attempted to load the following shells" << preferredShells;
     }
+
+    QWindowSystemInterfacePrivate::TabletEvent::setPlatformSynthesizesMouse(false);
 }
 
 QWaylandInputDevice *QWaylandIntegration::createInputDevice(QWaylandDisplay *display, int version, uint32_t id)
@@ -459,6 +448,42 @@ void QWaylandIntegration::initializeInputDeviceIntegration()
     }
 }
 
+void QWaylandIntegration::reconfigureInputContext()
+{
+    if (!mDisplay) {
+        // This function can be called from QWaylandDisplay::registry_global() when we
+        // are in process of constructing QWaylandDisplay. Configuring input context
+        // in that case is done by calling reconfigureInputContext() from QWaylandIntegration
+        // constructor, after QWaylandDisplay has been constructed.
+        return;
+    }
+
+    const QString &requested = QPlatformInputContextFactory::requested();
+    if (requested == QLatin1String("qtvirtualkeyboard"))
+        qCWarning(lcQpaWayland) << "qtvirtualkeyboard currently is not supported at client-side,"
+                                   " use QT_IM_MODULE=qtvirtualkeyboard at compositor-side.";
+
+    if (requested.isNull())
+        mInputContext.reset(new QWaylandInputContext(mDisplay.data()));
+    else
+        mInputContext.reset(QPlatformInputContextFactory::create(requested));
+
+    const QString defaultInputContext(QStringLiteral("compose"));
+    if ((!mInputContext || !mInputContext->isValid()) && requested != defaultInputContext)
+        mInputContext.reset(QPlatformInputContextFactory::create(defaultInputContext));
+
+#if QT_CONFIG(xkbcommon)
+    QXkbCommon::setXkbContext(mInputContext.data(), mDisplay->xkbContext());
+#endif
+
+    // Even if compositor-side input context handling has been requested, we fallback to
+    // client-side handling if compositor does not provide the text-input extension. This
+    // is why we need to check here which input context actually is being used.
+    mDisplay->mUsingInputContextFromCompositor = qobject_cast<QWaylandInputContext *>(mInputContext.data());
+
+    qCDebug(lcQpaWayland) << "using input method:" << inputContext()->metaObject()->className();
+}
+
 QWaylandShellIntegration *QWaylandIntegration::createShellIntegration(const QString &integrationName)
 {
     if (QWaylandShellIntegrationFactory::keys().contains(integrationName)) {
